@@ -1,3 +1,6 @@
+extern crate glob;
+
+use glob::glob;
 use std::env;
 use std::fs;
 use std::io::{self, BufRead, Write};
@@ -20,7 +23,11 @@ fn get_test_path(file_name: &str) -> io::Result<path::PathBuf> {
 /// the seed test cases that we pass to AFL.rs assert (including the failing
 /// test cases historically found by AFL.rs).
 fn generate_sanity_tests_from_afl_seeds() -> io::Result<()> {
-    println!("cargo:rerun-if-changed=in/*");
+    for entry in glob("./in/*").expect("should read glob pattern") {
+        if let Ok(path) = entry {
+            println!("cargo:rerun-if-changed={}", path.display());
+        }
+    }
     println!("cargo:rerun-if-changed=tests/afl_seeds.rs");
 
     let test_path = try!(get_test_path("afl_seeds.rs"));
@@ -68,7 +75,7 @@ fn test_afl_seed_{}() {{
 // Ratcheting number that is increased as more libiberty tests start
 // passing. Once they are all passing, this can be removed and we can enable all
 // of them by default.
-const LIBIBERTY_TEST_THRESHOLD: usize = 58;
+const LIBIBERTY_TEST_THRESHOLD: usize = 60;
 
 /// Read `tests/libiberty-demangle-expected`, parse its input mangled symbols,
 /// and expected output demangled symbols, and generate test cases for them.
@@ -85,7 +92,11 @@ fn generate_compatibility_tests_from_libiberty() -> io::Result<()> {
     let _ = fs::remove_file(&test_path);
     let mut test_file = try!(fs::File::create(test_path));
 
-    try!(writeln!(&mut test_file, "extern crate cpp_demangle;"));
+    try!(writeln!(&mut test_file, "
+extern crate cpp_demangle;
+extern crate diff;
+use std::fmt::Write;
+"));
 
     let libiberty_tests = try!(get_test_path("libiberty-demangle-expected"));
     let libiberty_tests = try!(fs::File::open(libiberty_tests));
@@ -147,6 +158,12 @@ fn generate_compatibility_tests_from_libiberty() -> io::Result<()> {
             continue;
         }
 
+        let cfg = if n <= LIBIBERTY_TEST_THRESHOLD {
+            ""
+        } else {
+            r###"#[cfg(feature = "run_libiberty_tests")]"###
+        };
+
         try!(writeln!(test_file,
                       r###"
 {}
@@ -159,18 +176,48 @@ fn test_libiberty_demangle_{}_() {{
         .expect("should parse mangled symbol");
 
     let expected = r#"{}"#;
-    let actual = format!("{{}}", sym);
+
+    let mut actual = String::new();
+    if let Err(e) = write!(&mut actual, "{{}}", sym) {{
+        panic!("Error while demangling '{{}}': {{}}",
+               String::from_utf8_lossy(mangled),
+               e);
+    }}
+
     println!("     Expect demangled symbol: {{}}", expected);
     println!("Actually demangled symbol as: {{}}", actual);
 
+    if expected != actual {{
+        println!("");
+        println!("Diff:");
+        println!("--- expected");
+        print!("+++ actual");
+
+        let mut last = None;
+        for cmp in diff::chars(expected, &actual) {{
+            match (last, cmp.clone()) {{
+                (Some(diff::Result::Left(_)), diff::Result::Left(_)) |
+                (Some(diff::Result::Both(..)), diff::Result::Both(..)) |
+                (Some(diff::Result::Right(_)), diff::Result::Right(_)) => {{}}
+
+                (_, diff::Result::Left(_))  => print!("\n-"),
+                (_, diff::Result::Both(..))  => print!("\n "),
+                (_, diff::Result::Right(_)) => print!("\n+"),
+            }};
+            match cmp.clone() {{
+                diff::Result::Left(c) |
+                diff::Result::Both(c, _) |
+                diff::Result::Right(c) => print!("{{}}", c),
+            }}
+            last = Some(cmp);
+        }}
+        println!("");
+    }}
+
     assert_eq!(expected, actual);
 }}
 "###,
-                      if n <= LIBIBERTY_TEST_THRESHOLD {
-                          ""
-                      } else {
-                          r###"#[cfg(feature = "run_libiberty_tests")]"###
-                      },
+                      cfg,
                       n,
                       mangled.trim(),
                       demangled.trim()));
